@@ -56,7 +56,7 @@ struct GvcMixerDialogPrivate
 {
         GvcMixerControl *mixer_control;
         GHashTable      *bars;        
-        GtkWidget       *notebook;
+        GtkWidget       *stack;
         GtkWidget       *output_bar;
         GtkWidget       *input_bar;
         GtkWidget       *input_level_bar;
@@ -774,9 +774,10 @@ active_input_update (GvcMixerDialog *dialog,
         adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (dialog->priv->input_bar)));
         g_signal_handlers_disconnect_by_func(adj, on_adjustment_value_changed, dialog);
 
-        gtk_label_set_label (GTK_LABEL(dialog->priv->selected_input_label),
-                             g_strdup_printf(_("Settings for %s"),
-                                             gvc_mixer_ui_device_get_description (active_input)));  
+        gchar *str = g_strdup_printf(_("Settings for %s"),
+                                      gvc_mixer_ui_device_get_description (active_input));
+        gtk_label_set_label (GTK_LABEL(dialog->priv->selected_input_label), str);
+        g_free (str);
 
         gvc_channel_bar_set_base_volume (GVC_CHANNEL_BAR (dialog->priv->input_bar),
                                          gvc_mixer_stream_get_base_volume (stream));
@@ -944,9 +945,11 @@ active_output_update (GvcMixerDialog *dialog,
         gtk_widget_set_sensitive (dialog->priv->output_bar, 
                                   TRUE);
         // Set the label accordingly
-	gtk_label_set_label (GTK_LABEL(dialog->priv->selected_output_label),
-			     g_strdup_printf(_("Settings for %s"),
-                                             gvc_mixer_ui_device_get_description (active_output)));
+        gchar *str = g_strdup_printf(_("Settings for %s"),
+                                     gvc_mixer_ui_device_get_description (active_output));
+        gtk_label_set_label (GTK_LABEL(dialog->priv->selected_output_label), str);
+        g_free (str);
+
         g_debug ("\n active_output_update %s \n", gvc_mixer_ui_device_get_description (active_output));
 
         GList* profiles = gvc_mixer_ui_device_get_profiles (active_output);
@@ -1116,7 +1119,8 @@ on_control_stream_added (GvcMixerControl *control,
                 && !gvc_mixer_stream_is_virtual (stream)
                 && g_strcmp0 (app_id, "org.gnome.VolumeControl") != 0
                 && g_strcmp0 (app_id, "org.PulseAudio.pavucontrol") != 0)
-                && g_strcmp0 (app_id, "org.Cinnamon") != 0) {
+                && g_strcmp0 (app_id, "org.Cinnamon") != 0 /* FIXME: org.Cinnamon.xx.xx should be unified */
+                && g_strcmp0 (app_id, "org.cinnamon") != 0) {
 
                 GtkWidget      *bar;
 
@@ -1774,14 +1778,7 @@ gvc_mixer_dialog_constructor (GType                  type,
         GtkWidget        *settings_box;
         GtkWidget        *box;
         GtkWidget        *sbox;
-        GtkWidget        *ebox;
         GtkWidget        *test_sound_box;
-        GSList           *streams;
-        GSList           *cards;
-        GSList           *l;
-        GvcMixerStream   *stream;
-        GvcMixerCard     *card;
-        GtkTreeSelection *selection;        
 
         object = G_OBJECT_CLASS (gvc_mixer_dialog_parent_class)->constructor (type, n_construct_properties, construct_params);
 
@@ -1800,19 +1797,23 @@ gvc_mixer_dialog_constructor (GType                  type,
                             alignment,
                             FALSE, FALSE, 0);
 
-        self->priv->notebook = gtk_notebook_new ();
+        self->priv->stack = gtk_stack_new ();
+        gtk_stack_set_transition_type (GTK_STACK (self->priv->stack),
+                                       GTK_STACK_TRANSITION_TYPE_SLIDE_LEFT_RIGHT);
+        gtk_stack_set_transition_duration (GTK_STACK (self->priv->stack), 150);
+
         gtk_box_pack_start (GTK_BOX (main_vbox),
-                            self->priv->notebook,
+                            self->priv->stack,
                             TRUE, TRUE, 0);
-        gtk_container_set_border_width (GTK_CONTAINER (self->priv->notebook), 5);
+        gtk_container_set_border_width (GTK_CONTAINER (self->priv->stack), 5);
 
         /* Output page */
         self->priv->output_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);        
         gtk_container_set_border_width (GTK_CONTAINER (self->priv->output_box), 12);
-        label = gtk_label_new (_("Output"));
-        gtk_notebook_append_page (GTK_NOTEBOOK (self->priv->notebook),
-                                  self->priv->output_box,
-                                  label);
+        gtk_stack_add_titled (GTK_STACK (self->priv->stack),
+                              self->priv->output_box,
+                              "output",
+                              _("Output"));
 
         box = gtk_frame_new (_("Play sound through"));
         gtk_widget_set_size_request (GTK_WIDGET (box), 310, -1);        
@@ -1942,10 +1943,10 @@ gvc_mixer_dialog_constructor (GType                  type,
         /* Input page */
         self->priv->input_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
         gtk_container_set_border_width (GTK_CONTAINER (self->priv->input_box), 12);
-        label = gtk_label_new (_("Input"));
-        gtk_notebook_append_page (GTK_NOTEBOOK (self->priv->notebook),
-                                  self->priv->input_box,
-                                  label);
+        gtk_stack_add_titled (GTK_STACK (self->priv->stack),
+                              self->priv->input_box,
+                              "input",
+                              _("Input"));
         box = gtk_frame_new (_("Record sound from"));
         gtk_widget_set_size_request (GTK_WIDGET (box), 310, -1);        
         label = gtk_frame_get_label_widget (GTK_FRAME (box));
@@ -2048,10 +2049,10 @@ gvc_mixer_dialog_constructor (GType                  type,
         /* Effects page */
         self->priv->sound_effects_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
         gtk_container_set_border_width (GTK_CONTAINER (self->priv->sound_effects_box), 12);
-        label = gtk_label_new (_("Sound Effects"));
-        gtk_notebook_append_page (GTK_NOTEBOOK (self->priv->notebook),
-                                  self->priv->sound_effects_box,
-                                  label);
+        gtk_stack_add_titled (GTK_STACK (self->priv->stack),
+                              self->priv->sound_effects_box,
+                              "effects",
+                              _("Sound Effects"));
 
         self->priv->effects_bar = create_bar (self, FALSE, TRUE);
         gvc_channel_bar_set_name (GVC_CHANNEL_BAR (self->priv->effects_bar),
@@ -2069,7 +2070,7 @@ gvc_mixer_dialog_constructor (GType                  type,
 
         add_sound_effect_selector (sounds_grid, 0, _("Starting Cinnamon:"), sound_settings, "login-file", "login-enabled", self);
         add_sound_effect_selector (sounds_grid, 1, _("Switching workspace:"), sound_settings, "switch-file", "switch-enabled", self);       
-        add_sound_effect_selector (sounds_grid, 2, _("Mapping windows:"), sound_settings, "map-file", "map-enabled", self);       
+        add_sound_effect_selector (sounds_grid, 2, _("Opening new windows:"), sound_settings, "map-file", "map-enabled", self);       
         add_sound_effect_selector (sounds_grid, 3, _("Closing windows:"), sound_settings, "close-file", "close-enabled", self);       
         add_sound_effect_selector (sounds_grid, 4, _("Minimizing windows:"), sound_settings, "minimize-file", "minimize-enabled", self);       
         add_sound_effect_selector (sounds_grid, 5, _("Maximizing windows:"), sound_settings, "maximize-file", "maximize-enabled", self);       
@@ -2085,10 +2086,10 @@ gvc_mixer_dialog_constructor (GType                  type,
         /* Applications */
         self->priv->applications_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
         gtk_container_set_border_width (GTK_CONTAINER (self->priv->applications_box), 12);
-        label = gtk_label_new (_("Applications"));
-        gtk_notebook_append_page (GTK_NOTEBOOK (self->priv->notebook),
-                                  self->priv->applications_box,
-                                  label);
+        gtk_stack_add_titled (GTK_STACK (self->priv->stack),
+                              self->priv->applications_box,
+                              "applications",
+                              _("Applications"));
         self->priv->no_apps_label = gtk_label_new (_("No application is currently playing or recording audio."));
         gtk_box_pack_start (GTK_BOX (self->priv->applications_box),
                             self->priv->no_apps_label,
@@ -2241,33 +2242,13 @@ gvc_mixer_dialog_new (GvcMixerControl *control)
         return GVC_MIXER_DIALOG (dialog);
 }
 
-enum {
-        PAGE_OUTPUT,
-        PAGE_INPUT,
-        PAGE_EVENTS,
-        PAGE_APPLICATIONS
-};
-
 gboolean
 gvc_mixer_dialog_set_page (GvcMixerDialog *self,
                            const char     *page)
 {
-        guint num;
-
         g_return_val_if_fail (self != NULL, FALSE);
 
-        num = PAGE_OUTPUT;
-
-        if (g_str_equal (page, "effects"))
-                num = PAGE_EVENTS;
-        else if (g_str_equal (page, "input"))
-                num = PAGE_INPUT;
-        else if (g_str_equal (page, "output"))
-                num = PAGE_OUTPUT;
-        else if (g_str_equal (page, "applications"))
-                num = PAGE_APPLICATIONS;
-
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (self->priv->notebook), num);
+        gtk_stack_set_visible_child_name (GTK_STACK (self->priv->stack), page);
 
         return TRUE;
 }
